#include "ledlib.h"

int map_file = 0;

#define SLCR_UNLOCK_OFFSET		(0x08)
#define APER_CLK_CTRL_OFFSET 	(0x12c)
#define MIO_PIN_BASE_ADDR 		(0x700)

/* each GPIO has a specific register for mux conf */
#define MIO_PIN_XX_REG(__base__, __pin__) \
	(__base__ + MIO_PIN_BASE_ADDR + (__pin__ << 2))
#define MIO_PIN_XX_TST(__pin__) \
	(MIO_PIN_BASE_ADDR + (__pin__ << 2))

unsigned char *open_slcr_and_unlock()
{
	const int base_addr1 = 0xF8000000;
	unsigned char *h = NULL;
	unsigned int page_addr, page_offset;
	unsigned page_size = sysconf(_SC_PAGESIZE);

	page_addr = base_addr1 & (~(page_size - 1));
	page_offset = base_addr1 - page_addr;
	if (page_offset != 0)
		return (NULL);
	printf("%x %x \n", page_size, page_addr);

	// mmap the device into memory 
	map_file = open("/dev/mem", O_RDWR | O_SYNC);
	h = mmap(NULL, page_size, PROT_READ | PROT_WRITE, MAP_SHARED, map_file,
		 page_addr);
	if (h < 0) {
		//printf("mmap pointer: %x\n", (int)(h));
		return (NULL);
	}

	// exemple baremetal pour debloquer acces SLCR registers
	// https://forums.xilinx.com/xlnx/attachments/xlnx/EDK/29780/1/helloworld.c
	*(unsigned int *)(h + SLCR_UNLOCK_OFFSET) = 0xDF0D;	// unlock
	// https://forums.xilinx.com/t5/Embedded-Linux/Zynq-mmap-GPIO/td-p/368601
	*(unsigned int *)(h + APER_CLK_CTRL_OFFSET) |= (1 << 22);	// activate clock
	return h;
}

unsigned char *red_gpio_init(int pin_num)
{
	const int base_addr2 = 0xe000a000;
	unsigned char *h = NULL;
	unsigned int page_addr, page_offset;
	unsigned page_size = sysconf(_SC_PAGESIZE);

#if 0
	page_addr = base_addr1 & (~(page_size - 1));
	page_offset = base_addr1 - page_addr;
	if (page_offset != 0)
		return (NULL);

	// mmap the device into memory 
	map_file = open("/dev/mem", O_RDWR | O_SYNC);
	h = mmap(NULL, page_size, PROT_READ | PROT_WRITE, MAP_SHARED, map_file,
		 page_addr);
	if (h < 0) {
		printf("mmap pointer: %x\n", (int)(h));
		return (NULL);
	}
#endif
	h = open_slcr_and_unlock();
	if (h == NULL)
		return NULL;

	printf("%x\n", MIO_PIN_XX_TST(pin_num));

	*(unsigned int *)(MIO_PIN_XX_REG(h, pin_num)) = 0 << 13 // disableRcvr
		| 0 << 12 // pull up disable
		| 3 << 9 // LVCMOS33
		| 1 << 8 // fast CMOS EDGE
		| 0 << 5 // gpio (input/output) mode
		| 0 << 3 // l3 mux
		| 0 << 2 // l2 mux
		| 0 << 1 // l1 mux
		| 0 << 0; // tri disable

	close(map_file);

	page_addr = base_addr2 & (~(page_size - 1));
	page_offset = base_addr2 - page_addr;
	if (page_offset != 0)
		return (NULL);

	// mmap the device into memory 
	map_file = open("/dev/mem", O_RDWR | O_SYNC);
	h = mmap(NULL, page_size, PROT_READ | PROT_WRITE, MAP_SHARED, map_file,
		 page_addr);
	if (h < 0) {
		//printf("mmap pointer: %x\n", (int)(h));
		return (NULL);
	}
	return (h);
}

void red_gpio_set_cfgpin(unsigned char *h, int num)
{				
	/* we must first configure pin as GPIO function in MIO_PIN_xx */

	// https://www.xilinx.com/support/documentation/user_guides/ug585-Zynq-7000-TRM.pdf p.1347
	uint16_t val = (1<<num);
	uint32_t reg = (((~val)&0xffff) << 16) | val;
	*(unsigned int *)(h + 0) = reg;	// GPIO programming sequence :p.386
	printf("%x \n", reg);
	*(unsigned int *)(h + 0x204) |= (1 << num);	// direction
	*(unsigned int *)(h + 0x208) |= (1 << num);	// output enable
}

void red_gpio_output(unsigned char *h, int num, int value)
{
	if (value != 0)
		*(unsigned int *)(h + 0x040) |= (1 << num);	// output value
	else
		*(unsigned int *)(h + 0x040) &= (~(1 << num));	// output value
}

void red_gpio_cleanup()
{
	close(map_file);
}
