library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.all;

Entity generateur_axi is 
    generic(
        id        : natural := 1;
		-- Parameters of Axi Slave Bus Interface S00_AXI
		C_S00_AXI_DATA_WIDTH	: integer	:= 32;
		C_S00_AXI_ADDR_WIDTH	: integer	:= 4;
	    MAX_VAL: natural := 3
    );
    port (
		-- signal
		output_o	: out std_logic;
		-- axi slave signals
		s00_axi_aclk	: in std_logic;
		s00_axi_reset	: in std_logic;
		s00_axi_awaddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_awprot  : in std_logic_vector(2 downto 0);
		s00_axi_awvalid	: in std_logic;
		s00_axi_awready	: out std_logic;
		s00_axi_wdata	: in std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_wvalid	: in std_logic;
		s00_axi_wready	: out std_logic;
		s00_axi_wstrb	: in std_logic_vector(3 downto 0);
		s00_axi_bresp	: out std_logic_vector(1 downto 0);
		s00_axi_bvalid	: out std_logic;
		s00_axi_bready	: in std_logic;
		s00_axi_araddr	: in std_logic_vector(C_S00_AXI_ADDR_WIDTH-1 downto 0);
		s00_axi_arprot  : in std_logic_vector(2 downto 0);
		s00_axi_arvalid	: in std_logic;
		s00_axi_arready	: out std_logic;
		s00_axi_rdata	: out std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
		s00_axi_rresp	: out std_logic_vector(1 downto 0);
		s00_axi_rvalid	: out std_logic;
		s00_axi_rready	: in std_logic
	);
end entity;

-----------------------------------------------------------------------
Architecture generateur_1 of generateur_axi is
    -- comm
    constant INTERNAL_ADDR_WIDTH : natural := 2;
    signal addr_s : std_logic_vector(INTERNAL_ADDR_WIDTH-1 downto 0);
    signal write_en_s, read_en_s : std_logic;
	signal prescaler_s			: std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
	signal max_period_s			: std_logic_vector(C_S00_AXI_DATA_WIDTH-1 downto 0);
begin
	gen_logic: entity work.generateur_logic
	port map (
		rst_i     => s00_axi_reset,
		clk_i     => s00_axi_aclk,
		output_o	=> output_o,
		max_period_i => max_period_s
	);

	gen_wb: entity work.generateur_comm
    generic map(
        id        => id,
		max_val		=> max_val,
        AXI_DATA_WIDTH => C_S00_AXI_DATA_WIDTH)
	port map (
		-- Syscon signals
		reset			=> s00_axi_reset,
		clk				=> s00_axi_aclk,
		-- comm signals
		addr_i 			=> addr_s,
		write_en_i 		=> write_en_s,
		writedata 		=> s00_axi_wdata,
		read_en_i 		=> read_en_s,
		readdata 		=> s00_axi_rdata,
		-- 
		max_period_o => max_period_s);

	-- Instantiation of Axi Bus Interface S00_AXI
	handle_comm : entity work.generateur_handCom
	generic map (C_S_AXI_DATA_WIDTH	=> C_S00_AXI_DATA_WIDTH,
		C_S_AXI_ADDR_WIDTH	=> C_S00_AXI_ADDR_WIDTH,
		INTERNAL_ADDR_WIDTH => INTERNAL_ADDR_WIDTH)
	port map (
		S_AXI_ACLK		=> s00_axi_aclk,
		S_AXI_RESET		=> s00_axi_reset,
		S_AXI_AWADDR	=> s00_axi_awaddr,
		S_AXI_AWVALID	=> s00_axi_awvalid,
		S_AXI_AWREADY	=> s00_axi_awready,
		S_AXI_WVALID	=> s00_axi_wvalid,
		S_AXI_WREADY	=> s00_axi_wready,
		S_AXI_BRESP	    => s00_axi_bresp,
		S_AXI_BVALID	=> s00_axi_bvalid,
		S_AXI_BREADY	=> s00_axi_bready,
		S_AXI_ARADDR	=> s00_axi_araddr,
		S_AXI_ARVALID	=> s00_axi_arvalid,
		S_AXI_ARREADY	=> s00_axi_arready,
		S_AXI_RRESP	    => s00_axi_rresp,
		S_AXI_RVALID	=> s00_axi_rvalid,
		S_AXI_RREADY	=> s00_axi_rready,
		read_en_o 		=> read_en_s,
		write_en_o 		=> write_en_s,
		addr_o 			=> addr_s);

end architecture generateur_1;
